//    Cube Viewer Object
//
//  A Java Cube Viewer Applet
//  by Charles W. Neville
//
//  Copyright November, 1999 by Charles W. Neville

// A Cube Viewer Object takes a cube in 3 dimensional world coordinates,
// converts to 3 dimensional viewing coordinates, and draws the a wire
// frame view of the cube.  It also contains text fields and buttons
// so the user may rotate the cube around any of the x, y and z world 
// coordinate axes.

// Because there is not much of a perspective effect when a cube is
// viewed close up, we use parallel projection rather than perspective
// projection in this demo.

// This demo is not fully functional.  As it stands, you can only
// rotate around the z world coordinate axis.  You have to add 
// rotation methods to each of the Point, Edge, Face and Cube 
// classes for rotation around the x and y world coordinate axes,
// and you have to call them appropriately in this demo.  Also, the
// cube is drawn up at the left hand corner of the screen.  You have
// to move the world coordinate origin to the screen center in viewing
// coordinates to make the cube appear where it belongs, in the
// center of the screen.

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import Point;
import Edge;
import Face;
import Cube;

public class CubeViewer extends Applet implements ActionListener {

  private Cube myCube;  // the cube we are going to view and rotate


  private Label rotXLabel, rotYLabel, rotZLabel;
                                      // for displaying what the
                                      // rotation angles refer to

  private TextField rotXField, rotYField, rotZField;
                                      // for entering the cube's
                                      // rotation angles

  private Button rotXButton, rotYButton, rotZButton;  
                                      // for changing the cube's
                                      // rotation angles

  // as usual, init builds the Applet Panel.  It also
  // creates and scales the cube we are going to rotate.
  public void init() {

    // labels and initialization string for spacing
    String spaces = "                ";
    Label spaceXLabel = new Label(spaces);
    Label spaceYLabel = new Label(spaces);
    Label spaceZLabel = new Label(spaces);

    myCube = new Cube();  // create new 1 by 1 by 1 pixel cube object
    myCube.scale(100);    // make it a 100 by 100 by 100 pixels cube

    // build Applet Panel

    // x axis rotation

    // informative label
    rotXLabel = new Label(" x angle (degrees) ");
                            // create new Label object with 
                            // given title
    add(rotXLabel);         // add Label to Applet Panel

    // for entering and displaying rotation around x axis
    rotXField = new TextField(4);  // create new TextField object
                                   // 4 characters wide
    add(rotXField);                // add TextField to Applet Panel

    // for rotation around x axis
    rotXButton = new Button(" rotate "); // create new Button  
                                         // object with given title
    add(rotXButton);                     // add Button to Applet Panel
    rotXButton.addActionListener(this);  // delegate ourselves as the
                                         // Button event processor

    // spacing
    add(spaceXLabel);

    // y axis rotation

    // informative label
    rotYLabel = new Label(" y angle (degrees) ");
    add(rotYLabel);

    // for entering and displaying rotation around y axis
    rotYField = new TextField(4);
    add(rotYField);

    // for rotation around y axis
    rotYButton = new Button(" rotate ");
    add(rotYButton);
    rotYButton.addActionListener(this);

    // spacing
    add(spaceYLabel);

    // z axis rotation

    // informative label
    rotZLabel = new Label(" z angle (degrees) ");
    add(rotZLabel);

    // for entering and displaying rotation around z axis
    rotZField = new TextField(4);
    add(rotZField);

    // for rotation around z axis
    rotZButton = new Button(" rotate ");
    add(rotZButton);
    rotZButton.addActionListener(this);

    // spacing
    add(spaceZLabel);
  } // init()

  // view an edge by drawing line between its endpoints.  Because
  // this Applet is a wire frame viewer, viewEdge is the Applet's
  // fundamental drawing method.
  private void viewEdge(Graphics g, Edge e) {

    double x1, y1, x2, y2;  // x, y world coordinates of starting
                            // and ending points of edge.
    double xPrime1, yPrime1, xPrime2, yPrime2;
                            // x, y viewing coordinates of starting
                            // and ending points of edge
    int xScreen1, yScreen1, xScreen2, yScreen2;
                            // x, y screen coordinates of starting
                            // and ending points of edge

    x1 = e.endpoints[Edge.first].coords[Point.x];
    y1 = e.endpoints[Edge.first].coords[Point.y];
    x2 = e.endpoints[Edge.last].coords[Point.x];
    y2 = e.endpoints[Edge.last].coords[Point.y];

    // you need to modify these so that world coordinate origin
    // is at (200, -200) in viewing coordinates (center of Applet 
    // Panel
    xPrime1 = x1;
    yPrime1 = y1;
    xPrime2 = x2;
    yPrime2 = y2;

    xScreen1 = (int) Math.round(xPrime1);
    yScreen1 = - (int) Math.round(yPrime1);
                         // make y viewing coordinate axis point up,
                         // not down in screen coordinates
    xScreen2 = (int) Math.round(xPrime2);
    yScreen2 = - (int) Math.round(yPrime2);
                         // make y viewing coordinate axis point up,
                         // not down in screen coordinates

    g.drawLine(xScreen1, yScreen1, xScreen2, yScreen2);
  } // viewEdge()

  // view a face by drawing its bounding edges
  private void viewFace(Graphics g, Face f) {

    for(int i = 0; i < (f.edges).length; i++) {

      viewEdge(g, f.edges[i]);
    } // for
  } // viewFace()

  // view the entire cube by drawing its bounding faces
  private void viewCube(Graphics g, Cube c) {

     for(int i = 0; i < (c.faces).length; i++) {

       viewFace(g, c.faces[i]);
     } // for
  } // viewCube()

  // Button event handler.  This is called by the Java run time
  // system every time one of the Buttons is clicked.
  public void actionPerformed(ActionEvent e) {

    double thetaX, thetaY, thetaZ;  // rotation angles

    if (e.getSource() == rotXButton) {
      thetaX = Integer.parseInt(rotXField.getText());
      // you put in the method call to rotate around the world 
      // coordinate x axis, and delete this remark when you do.
    }
    else if (e.getSource() == rotYButton) {
      thetaY = Integer.parseInt(rotYField.getText());
      // you put in the method call to rotate around the world 
      // coordinate x axis, and delete this remark when you do.
    }
    else if (e.getSource() == rotZButton) {
      thetaZ = Integer.parseInt(rotZField.getText());
                                    // assume user enters an integer 
                                    // number of degrees to rotate
      thetaZ = thetaZ*Math.PI/180;  // convert to radians
      myCube.rotZ(thetaZ);          // rotate cube around the world 
                                    // coordinate z axis
    }

    repaint();
  } // actionPerformed();

  // this is called by the Java runtime system when the Applet Panel
  // needs to be repainted (redrawn).  It is where we do the actual
  // drawing.
  public void paint(Graphics g) {

    g.setColor(Color.green);       // draw in vibrant green
    g.drawLine(0, 100, 400, 100);  // green dividing line between
                                   // Applet controls and viewscreen
     
    viewCube(g, myCube);           // draw the cube in its new
                                   // orientation!

    g.setColor(Color.black);       // drawing color back to black
  } // paint()

} // class CubeViewer
